%
%
%  Find Spots Close to Filaments for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Find Spots Close to Filaments" icon="Matlab" tooltip="Find spots close to filament.">
%          <Command>MatlabXT::XTSpotsCloseToFilaments(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Find Spots Close to Filaments" icon="Matlab" tooltip="Find spots close to filament.">
%            <Command>MatlabXT::XTSpotsCloseToFilaments(%i)</Command>
%          </Item>
%        </SurpassComponent>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Find Spots Close to Filaments" icon="Matlab" tooltip="Find spots close to filament.">
%            <Command>MatlabXT::XTSpotsCloseToFilaments(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%
%  
%  Description:
%
%   The user has to create a Spots and a Filaments component. 
%   This XTension segments the spots into two groups: Spots which are 
%       close to the filamentous structure will be colored red, the others 
%       blue. 
%   This might help to count spines.
%
%   Usage Example:
%       - Load "C2CZOSmall.ims".
%       - Create Spots (ch1, 0.5um, on, 0.5um, thresh.10)
%       - Create Filaments (default values, ratio branch / trunk = 3)
%       - Select tools tab from spots component
%       - Start "Find Spots close to a Filaments"
%       - Ratio = 0.7
%
%

function XTSpotsCloseToFilaments(aImarisApplicationID)

if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

%
% GUI - user interactions
%

% the user has to create a scene with a filament and some spots
if isequal(vImarisApplication.GetSurpassScene, [])
    msgbox('Please create a Filaments and some Spots!');
    return;
end

% THE filament and THE spot object
vFilaments = vImarisApplication.GetFactory.ToFilaments(vImarisApplication.GetSurpassSelection);
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);

% find a filament and some spots from the surpass scene
vSurpassScene = vImarisApplication.GetSurpassScene;
for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if isequal(vSpots, [])
        if vImarisApplication.GetFactory.IsSpots(vDataItem)
            vSpots = vImarisApplication.GetFactory.ToSpots(vDataItem);
        end
    end
    if isequal(vFilaments, [])
        if vImarisApplication.GetFactory.IsFilaments(vDataItem)
            vFilaments = vImarisApplication.GetFactory.ToFilaments(vDataItem);
        end
    end
end

% did we find the filament?
if isequal(vFilaments, [])
    msgbox('Please create a Filaments!');
    return;
end

% did we find the spots?
if isequal(vSpots, [])
    msgbox('Please create some Spots!');
    return;
end

% ask user for a distance
vAnswer = inputdlg({ ...
  sprintf('"%s" - "%s"\nRatio of distance (Spot - FilamentsCenter) to Filaments radius', ...
  char(vFilaments.GetName), char(vSpots.GetName))}, 'Find Spots close to Filaments', 1, {'0.9'});
if isempty(vAnswer)
    return;
end
vRatioLimit = str2double(vAnswer);

%
% ALGORITHM - get spot coordinates and filament vertex coordinates from
% imaris and compute the distance between each pair. if the shortest
% distance of a spot to a filament vertex is shorter than our limit, the
% spot is close.
%

% get the spot coordinates
vSpotsPosXYZ = vSpots.GetPositionsXYZ;
vSpotsRadius = vSpots.GetRadiiXYZ;
vSpotsPosT = vSpots.GetIndicesT;
vSpotsCount = size(vSpotsPosXYZ, 1);

% get the filament vertices
vCount = vFilaments.GetNumberOfFilaments;
vValues = vFilaments.GetFilamentsList(0:vCount-1);
vFilamentPosXYZ = vValues.mPositionsXYZ;
vFilamentRadius = vValues.mRadii;

% bool vector for result (A is close, B is far)
vIsClose = false(vSpotsCount, 1);

% find close and far spots
vProgressDisplay = waitbar(0, 'Finding close Spots ...');
for vSpotsIndex = 1:vSpotsCount
  vDistX = vFilamentPosXYZ(:, 1) - vSpotsPosXYZ(vSpotsIndex, 1);
  vDistY = vFilamentPosXYZ(:, 2) - vSpotsPosXYZ(vSpotsIndex, 2);
  vDistZ = vFilamentPosXYZ(:, 3) - vSpotsPosXYZ(vSpotsIndex, 3);
  vDist = sqrt(sum([vDistX, vDistY, vDistZ].^2, 2));
  vMinRatio = min(vDist ./ vFilamentRadius);
  vIsClose(vSpotsIndex) = vMinRatio < vRatioLimit;
  waitbar(vSpotsIndex / vSpotsCount);
end
close(vProgressDisplay);

% hide original spots
vSpots.SetVisible(false);

% add new spots (close)
vSpotsA = vImarisApplication.GetFactory.CreateSpots;
% The radius can be setted to zero because it is overwritten on the next line
% Only an array with the right size is needed
vSpotsA.Set(vSpotsPosXYZ(vIsClose,:), vSpotsPosT(vIsClose), zeros(sum(vIsClose),1));
vSpotsA.SetRadiiXYZ(vSpotsRadius(vIsClose,:));
vSpotsA.SetName(sprintf('%s closer than %0.3f of radius', char(vSpots.GetName), vRatioLimit));
vSpotsA.SetColorRGBA(255);
vImarisApplication.GetSurpassScene.AddChild(vSpotsA, -1);

% add new spots (far)
vSpotsB = vImarisApplication.GetFactory.CreateSpots;
vIsFar = ~vIsClose;
vSpotsB.Set(vSpotsPosXYZ(vIsFar,:), vSpotsPosT(vIsFar),  zeros(sum(vIsFar),1));
vSpotsB.SetRadiiXYZ(vSpotsRadius(vIsFar,:));
vSpotsB.SetName(sprintf('%s farther than %0.3f of radius', char(vSpots.GetName), vRatioLimit));
vSpotsB.SetColorRGBA(255*256*256);
vImarisApplication.GetSurpassScene.AddChild(vSpotsB, -1);
